/*
  xtfield.h
  Definition for XtInputStringField and XtInputNumericField

  Copyright (c) 1996-99 Dragos Acostachioaie
  Copyright (c) 1999 Udo Kreckel
  Documentation Copyright (c) 1998-99 Dragos Acostachioaie

  This file is part of Xterminal library.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the license, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _XTFIELD_H
#define _XTFIELD_H

#include <xtscreen.h>
#include <xobject.h>

/**
    @memo Field to read strings
*/
class XtInputStringField : public XObject {
 public:

  /** Constructs a new XtInputStringField.
      @param __r field bounds
      @param __DefaultString default string
      @param __MaxLen maximum string length
      @param __Flags options, which can be:
         <p><b>FIELD_HIDDEN</b> - the output should be hidden
	 <p><b>FIELD_READONLY</b> - the content cannot be changed
	 <p><b>FIELD_JUSTIFIED_RIGHT</b> - the field is right-justified
  */
  XtInputStringField(XRect __r, char *__DefaultString, unsigned __MaxLen, unsigned long __Flags);

  /// Destructs the object's instance
  virtual ~XtInputStringField();

  /// Draws the object
  virtual void Draw();

  /// Returns the current value
  virtual char *GetValue();

  /// Returns the default value
  virtual char *GetDefaultValue();

  /// Process keyboard events
  virtual bool ProcessKeyboardEvent(XKeyboardEvent *__Event);

  /// Process messages
  virtual bool ProcessMessage(XMessage *__Event);

  /// Process mouse events
  virtual bool ProcessMouseEvent(XMouseEvent *__Event);

  /// Sets the object's state
  virtual void SetState(unsigned long __Mask, bool __BoolValue);

  /// Sets the default value
  virtual void SetValue(char *__DefaultString);

 protected:
  char *DefaultString, *CurrString;
  unsigned FieldLength, StringLength, MaxLen, CP, curX, curY;
  unsigned LineLength, LineCount, StringPos;
  char bchar;
  bool InsertMode, InputText;
  unsigned long Flags;
};

#define FIELD_HIDDEN		0x0001
#define FIELD_READONLY		0x0002
#define FIELD_JUSTIFIED_RIGHT	0x0004

/**
    Field to read numeric values
*/
class XtInputNumericField : public XtInputStringField {
 public:

  /** Constructs a new XtInputNumericField.
      @param __r field bounds
      @param __DefaultNumber default number
      @param __IntegerDigits maximum integer digits allowed
      @param __DecimalDigits maximum decimal digits allowed
      @param __Flags options, which can be:
	 <p><b>FIELD_READONLY</b> - the content cannot be changed
	 <p><b>FIELD_JUSTIFIED_RIGHT</b> - the field is right-justified
         <p><b>ALLOW_NEGATIVE_VALUES</b> - to allow entering negative values
  */
  XtInputNumericField(XRect __r, double __DefaultNumber, unsigned char __IntegerDigits, unsigned char __DecimalDigits, unsigned long __Flags);

  virtual bool ProcessKeyboardEvent(XKeyboardEvent *__Event);

  /// Sets the default value
  virtual void SetValue(char *__DefaultString);

  /// Sets the default value
  virtual void SetValue(double __DefaultNumber);

 protected:
  unsigned short IntegerDigits, DecimalDigits;
};

#define ALLOW_NEGATIVE_VALUES	0x0010

// Palette entries
#define P_FIELD_LEN	6

#define P_FIELD_TEXT	0x01
#define P_FIELD_DEFAULT	0x03
#define P_FIELD_FOCUSED	0x05

// Default palette
#define P_FIELD 	"\x07\x00\x00\x06\x07\x00\x07\x00\x10\x06\x07\x00"

#endif
