/*
  xtlist.h
  Definition for XtList

  Copyright (c) 1998-99 Dragos Acostachioaie

  This file is part of Xterminal library.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the license, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _XTLIST_H
#define _XTLIST_H

#include <xobject.h>
#include <xtbar.h>

// List Item definition
struct LItem {
  char *Text;
  int Value;
  LItem *Next;
};

LItem *NewLItem(char *__Text, int __Value, LItem *__ANextItem);
LItem *NewList(char **__Text, unsigned __Count);

/**
    @memo Selectable list of items
*/
class XtList : public XObject {
 public:

  /** Constructs a new XtList.
      @param __r list bounds
      @param __Flags options, which can be:
          <p><b>SELECTABLE_ITEMS</b> - to allow selection (enable/disable) of items
	  <p><b>CIRCULAR_SELECTION</b> - make the list a circular one
	  <p><b>HIDE_DISABLED_LIST_ITEMS</b> - hide the items that are disabled
      @param __Items linked list of items:
          <p>struct LItem {<br>
               char *Text;<br>
               int Value;<br>
               LItem *Next;<br>
             };<br>
          <p>This list can be constructed using the following 2 functions:
	  <p>LItem *NewLItem(char *__Text, int __Value, LItem *__ANextItem)
	     which builds the entire list, and:
	  <p>LItem *NewList(char **__Text, unsigned __Count)
	     which builds from a list of char*, containing __Count items,
	     a linked list of items
  */
  XtList(XRect __r, unsigned long __Flags, LItem *__Items);

  /// Destructs the object's instance
  ~XtList();

  /// Draws the object
  virtual void Draw();

  /// Returns the items number
  virtual unsigned GetItemsNumber();

  /** Returns the value of the specified item number.
      @param __Item item number, 0 = current item, >0 = item's value
  */
  virtual int GetValue(unsigned __Item);

  /// Returns the list content
  virtual LItem* GetValue();

  /// Initialize the associated scroll bar
  virtual void InitScrollBar();

  /// Process keyboard events
  virtual bool ProcessKeyboardEvent(XKeyboardEvent *__Event);

  /// Process message events
  virtual bool ProcessMessage(XMessage *__Event);

  /// Process mouse events
  virtual bool ProcessMouseEvent(XMouseEvent *__Event);

  /** "Runs" the list: waits for events until an item is choosed or the
      dialog is closed
  */
  virtual int Run();

  /// Sets the object's state
  virtual void SetState(unsigned long __Mask, bool __BoolValue);

  /// Sets a new list content
  virtual void SetValue(LItem *__Items);

  /** Sets the value of the specified item number.
      @param __Item item number, 0 = current item, >0 = item's value
      @param __Value value
  */
  virtual void SetValue(unsigned __Item, unsigned __Value);

 protected:
  XtScrollBar *ScrollBar;
  unsigned long Flags;
  LItem *CurrIndex;
  unsigned CurrItem, ItemsNo, CP, CurY;
  XPoint OldCursorPos;
  LItem *Items;
};

#define SELECTABLE_ITEMS		0x0001
#define CIRCULAR_SELECTION		0x0002
#define SKIP_DISABLED_LIST_ITEMS	0x0004
#define HIDE_DISABLED_LIST_ITEMS	0x0008

#define ITEM_DISABLED	0x0001
#define ITEM_SELECTED	0x0002

// Palette entries
#define P_LIST_LEN			8

#define P_LIST_SURFACE			0x01
#define P_LIST_DISABLED			0x03
#define P_LIST_SELECTED			0x05
#define P_LIST_SELECTED_DISABLED	0x07

// Default palette
#define P_LIST 		"\x00\x07\x40\x07\x47\x06\x40\x06\x00\x07\x80\x07\x10\x07\x50\x07"

#endif
